
!ContextPart class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 9/6/2012 22:08'!
newForMethod: aMethod
	<omniUnenforced>
	"This is the only method for creating new contexts, other than primitive cloning.
	Any other attempts, such as inherited methods like shallowCopy, should be
	avoided or must at least be rewritten to determine the proper size from the
	method being activated.  This is because asking a context its size (even basicSize!!)
	will not return the real object size but only the number of fields currently
	accessible, as determined by stackp."

	^ super basicNew: aMethod frameSize! !

!ContextPart methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 21:56'!
canHandleSignal: exception
	"Sent to handler (on:do:) contexts only.  If my exception class (first arg) handles exception then return true, otherwise forward this message to the next handler context.  If none left, return false (see nil>>canHandleSignal:)"

	<omniUnenforced>
	^ (((self tempAt: 1) handles: exception) and: [self tempAt: 3])
		or: [self nextHandlerContext canHandleSignal: exception].
! !

!ContextPart methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 21:57'!
handleSignal: exception
	"Sent to handler (on:do:) contexts only.  If my exception class (first arg) handles exception then execute my handle block (second arg), otherwise forward this message to the next handler context.  If none left, execute exception's defaultAction (see nil>>handleSignal:)."

	<omniUnenforced>
	| val |
	(((self tempAt: 1) handles: exception) and: [self tempAt: 3]) ifFalse: [
		^ self nextHandlerContext handleSignal: exception].

	exception privHandlerContext: self contextTag.
	self tempAt: 3 put: false.  "disable self while executing handle block"
	val := [(self tempAt: 2) cull: exception]
		ensure: [self tempAt: 3 put: true].
	self return: val.  "return from self if not otherwise directed in handle block"
! !

!SmallInteger methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 15:38'!
highBit
	<omniUnenforced>
	"Answer the index of the high order bit of the receiver, or zero if the  
	receiver is zero. Raise an error if the receiver is negative, since  
	negative integers are defined to have an infinite number of leading 1's 
	in 2's-complement arithmetic. Use >>highBitOfMagnitude if you want to 
	get the highest bit of the magnitude."
	self < 0 ifTrue: [^ self error: 'highBit is not defined for negative integers'].
	^ self highBitOfPositiveReceiver! !

!SmallInteger methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 15:38'!
highBitOfPositiveReceiver
	| shifted bitNo |
	<omniUnenforced>
	"Answer the index of the high order bit of the receiver, or zero if the 
	receiver is zero. Receiver has to be positive!!"
	shifted := self.
	bitNo := 0.
	[shifted < 65536]
		whileFalse: 
			[shifted := shifted bitShift: -16.
			bitNo := bitNo + 16].
	shifted < 256
		ifFalse: 
			[shifted := shifted bitShift: -8.
			bitNo := bitNo + 8].
		
	"The high bits table can be obtained with:
	(1 to: 8) inject: #[0] into: [:highBits :rank | highBits , (highBits collect: [:e | rank])]."
	^bitNo + ( #[0 1 2 2 3 3 3 3 4 4 4 4 4 4 4 4 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 6 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 7 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8 8] at: shifted + 1)! !

!SmallInteger methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 15:44'!
lowBit
	<omniUnenforced>
	" Answer the index of the low order one bit.
		2r00101000 lowBit       (Answers: 4)
		2r-00101000 lowBit      (Answers: 4)
	  First we skip bits in groups of 8, then do a lookup in a table.
	  While not optimal, this is a good tradeoff; long
	  integer #lowBit always invokes us with bytes."
	| n result lastByte |
	n := self.
	n = 0 ifTrue: [ ^ 0 ].
	result := 0.
	[(lastByte := n bitAnd: 16rFF) = 0]
		whileTrue: [
			result := result + 8.
			n := n bitShift: -8 ].

	"The low bits table can be obtained with:
	((1 to: 8) inject: #[1] into: [:lowBits :rank | (lowBits copy at: 1 put: lowBits first + 1; yourself) , lowBits]) allButFirst."
	^result + ( #[1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 5 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 6 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 5 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 7 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 5 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 6 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 5 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 8 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 5 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 6 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 5 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 7 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 5 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 6 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1 5 1 2 1 3 1 2 1 4 1 2 1 3 1 2 1] at: lastByte)! !

!SmallInteger methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 10:57'!
quo: aNumber 
	"Primitive. Divide the receiver by the argument and answer with the 
	result. Round the result down towards zero to make it a whole integer. 
	Fail if the argument is 0 or is not a SmallInteger. Optional. See Object 
	documentation whatIsAPrimitive."
	<omniUnenforced>
	<primitive: 13>
	aNumber = 0 ifTrue: [^ (ZeroDivide dividend: self) signal].
	(aNumber isMemberOf: SmallInteger)
		ifFalse: [^ super quo: aNumber].
	(aNumber = -1 and: [self = self class minVal])
		ifTrue: ["result is aLargeInteger" ^ self negated].
	self primitiveFailed! !

!LargePositiveInteger methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 15:43'!
highBit
	<omniUnenforced>
	"Answer the index of the high order bit of the receiver, or zero if the  
	receiver is zero. Raise an error if the receiver is negative, since  
	negative integers are defined to have an infinite number of leading 1's 
	in 2's-complement arithmetic. Use >>highBitOfMagnitude if you want to  
	get the highest bit of the magnitude."
	^ self highBitOfMagnitude! !

!LargePositiveInteger methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 10:58'!
quo: anInteger 
	"Primitive. Divide the receiver by the argument and return the result.
	Round the result down towards zero to make it a whole integer. Fail if
	the argument is 0. Fail if either the argument or the result is not a
	SmallInteger or a LargePositiveInteger less than 2-to-the-30th (1073741824). Optional. See
	Object documentation whatIsAPrimitive."
	
	<omniUnenforced>
	<primitive: 33>
	^super quo: anInteger! !

!Character class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 5/7/2013 20:02'!
digitValue: x
	"We consider this as an operation on an immutable table... the unenforced here is an optimization on a global class"
	<omniUnenforced>
	
	"Answer the Character whose digit value is x. For example, answer $9 for 
	x=9, $0 for x=0, $A for x=10, $Z for x=35."

	| index |
	index := x asInteger.
	^CharacterTable at: 
		(index < 10
			ifTrue: [48 + index]
			ifFalse: [55 + index])
		+ 1! !

!Character class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 4/30/2013 17:23'!
value: anInteger 
	"We consider this an operation on an immutable table... the unenforced here is an optimization on a global class"
	<omniUnenforced>
	"Answer the Character whose value is anInteger."
	
	anInteger negative ifTrue:[self error: 'Characters expects a positive value.'].
	anInteger > 255 ifTrue: [^self basicNew setValue: anInteger].
	^ CharacterTable at: anInteger + 1.
! !

!Character methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:06'!
asInteger
	"Answer the value of the receiver."
	<omniUnenforced>
	^value! !

!Character methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:06'!
asLowercase
	"If the receiver is uppercase, answer its matching lowercase Character."
	"A tentative implementation.  Eventually this should consult the Unicode table."
	<omniUnenforced>
	| v |
	v := self charCode.
	(((8r101 <= v and: [v <= 8r132]) or: [16rC0 <= v and: [v <= 16rD6]]) or: [16rD8 <= v and: [v <= 16rDE]])
		ifTrue: [^ Character value: value + 8r40]
		ifFalse: [^ self]! !

!Character methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:04'!
asciiValue
	"Answer the value of the receiver that represents its ascii encoding."
	<omniUnenforced>
	^value! !

!Character methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:06'!
charCode
	<omniUnenforced>
	^ (value bitAnd: 16r3FFFFF).
! !

!Character methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 5/7/2013 20:09'!
digitValue
	"We consider this as an operation on an immutable table... the unenforced here is an optimization on a global class"
	<omniUnenforced>
	
	"Answer 0-9 if the receiver is $0-$9, 10-35 if it is $A-$Z, and < 0
	otherwise. This is used to parse literal numbers of radix 2-36."

	"slow version
	| digitValue |
	(digitValue := ('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ' indexOf: self) - 1) >= 0
		ifTrue: [ ^digitValue ].
	^ (EncodedCharSet charsetAt: self leadingChar) digitValueOf: self. "
	
	value > 16rFF ifTrue: [^(EncodedCharSet charsetAt: self leadingChar) digitValueOf: self].
	^DigitValues at: 1 + value! !

!Character methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:05'!
isOctetCharacter
	<omniUnenforced>
	^ value < 256.
! !

!Process class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 9/6/2012 21:51'!
forContext: aContext priority: anInteger 
	"Answer an instance of me that has suspended aContext at priority 
	anInteger."
	<omniUnenforced>
	| newProcess |
	newProcess := self new.
	newProcess suspendedContext: aContext.
	newProcess priority: anInteger.
	^newProcess! !

!Exception methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 22:08'!
resumeUnchecked: resumptionValue
	"Return resumptionValue as the value of #signal, unless this was called after an #outer message, then return resumptionValue as the value of #outer."
	<omniUnenforced>
	| ctxt |
	outerContext 
		ifNil: [
			signalContext return: resumptionValue ] 
		ifNotNil: [
			ctxt := outerContext.
			outerContext := ctxt tempAt: 1. "prevOuterContext in #outer"
			ctxt return: resumptionValue ]
! !

!Number methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 10:57'!
quo: aNumber
	<omniUnenforced>
	"Integer quotient defined by division with truncation toward zero. -9 quo: 
	4 = -2, -0.9 quo: 0.4 = -2. rem: answers the remainder from this division."

	^(self / aNumber) truncated! !

!Behavior methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:17'!
basicNew
	"Primitive. Answer an instance of the receiver (which is a class) with no 
	indexable variables. Fail if the class is indexable. Essential. See Object 
	documentation whatIsAPrimitive."

	<primitive: 70>
	<omniReplacement: #omniBasicNew>
	self isVariable ifTrue: [ ^ self basicNew: 0 ].
	"space must be low"
	OutOfMemory signal.
	^ self basicNew  "retry if user proceeds"! !

!Behavior methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:17'!
basicNew: sizeRequested 
	"Primitive. Answer an instance of this class with the number
	of indexable variables specified by the argument, sizeRequested.
	Fail if this class is not indexable or if the argument is not a
	positive Integer, or if there is not enough memory available. 
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 71>
	<omniReplacement: #omniBasicNew:>
	self isVariable ifFalse:
		[self error: self printString, ' cannot have variable sized instances'].
	(sizeRequested isInteger and: [sizeRequested >= 0]) ifTrue:
		["arg okay; space must be low."
		OutOfMemory signal.
		^ self basicNew: sizeRequested  "retry if user proceeds"].
	self primitiveFailed! !

!Behavior methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:15'!
isBytes
	"Answer whether the receiver has 8-bit instance variables."
	<omniUnenforced>
	^ self instSpec >= 8! !

!ByteArray methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:10'!
replaceFrom: start to: stop with: replacement startingAt: repStart
	"Primitive. This destructively replaces elements from start to stop in the receiver starting at index, repStart, in the collection, replacement. Answer the receiver. Range checks are performed in the primitive only. Optional. See Object documentation whatIsAPrimitive."
	
	<primitive: 105>
	<omniReplacement: #ReplaceFrom:to:with:startingAt:>
	super replaceFrom: start to: stop with: replacement startingAt: repStart! !

!Semaphore methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 14:08'!
signal
	"Primitive. Send a signal through the receiver. If one or more processes 
	have been suspended trying to receive a signal, allow the first one to 
	proceed. If no process is waiting, remember the excess signal. Essential. 
	See Object documentation whatIsAPrimitive."

	<primitive: 85>
	<omniUnenforced>
	self primitiveFailed

	"self isEmpty    
		ifTrue: [excessSignals := excessSignals+1]    
		ifFalse: [Processor resume: self removeFirstLink]"

! !

!WriteStream methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:08'!
nextPut: anObject
	"Primitive. Insert the argument at the next position in the Stream
	represented by the receiver. Fail if the collection of this stream is not an
	Array or a String. Fail if the stream is positioned at its end, or if the
	position is out of bounds in the collection. Fail if the argument is not
	of the right type for the collection. Optional. See Object documentation
	whatIsAPrimitive."

	<primitive: 66>
	<omniReplacement: #NextPut:>
	((collection class == ByteString) and: [
		anObject isCharacter and:[anObject isOctetCharacter not]]) ifTrue: [
			collection := (WideString from: collection).
			^self nextPut: anObject.
	].
	position >= writeLimit
		ifTrue: [^ self pastEndPut: anObject]
		ifFalse: 
			[position := position + 1.
			^collection at: position put: anObject]! !

!Array methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:09'!
replaceFrom: start to: stop with: replacement startingAt: repStart
	"Primitive. This destructively replaces elements from start to stop in the receiver starting at index, repStart, in the collection, replacement. Answer the receiver. Range checks are performed in the primitive only. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 105>
	<omniReplacement: #ReplaceFrom:to:with:startingAt:>
	super replaceFrom: start to: stop with: replacement startingAt: repStart! !

!MethodContext methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 22:08'!
tempAt: index 
	"Answer the value of the temporary variable whose index is the 
	 argument, index.  Primitive. Assumes receiver is indexable. Answer the
	 value of an indexable element in the receiver. Fail if the argument index
	 is not an Integer or is out of bounds. Essential. See Object documentation
	 whatIsAPrimitive.  Override the default at: primitive to give latitude to the
	 VM in context management."
	
	<omniUnenforced>
	<primitive: 210>
	^self at: index! !

!MethodContext methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 22:07'!
tempAt: index put: value 
	"Store the argument, value, as the temporary variable whose index is the 
	 argument, index.  Primitive. Assumes receiver is indexable. Answer the
	 value of an indexable element in the receiver. Fail if the argument index
	 is not an Integer or is out of bounds. Essential. See Object documentation
	 whatIsAPrimitive.  Override the default at:put: primitive to give latitude to
	 the VM in context management."

	<omniUnenforced>
	<primitive: 211>
	^self at: index put: value! !

!Halt class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:36'!
halt
	"backward compatible method with self halt"
	<omniUnenforced>
	self signal! !

!Halt class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:36'!
halt: aString
	"backward compatible method with self halt:"
	<omniUnenforced>
	self signal: aString! !

!Halt class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:36'!
if: condition
	"This is the typical message to use for inserting breakpoints during 
	debugging.
	
	The argument can be one of the following:
		- a block: if the Block has one arg, the calling object is bound to that.
		- an expression
 		- a selector: Halt if found in the call chain"
	<omniUnenforced>
	condition isSymbol ifTrue: [ ^ self haltIfCallChainContains: condition ].
	condition isBlock ifTrue: [ ^ self haltIfBlockWithCallingObject: condition].
	condition ifTrue: [self signal].! !

!Halt class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:36'!
now
	<omniUnenforced>
	self signal.! !

!Halt class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 13:36'!
now: aString
	"set a breakpoint with some explanation"
	<omniUnenforced>
	self signal: aString! !

!Integer methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 15:44'!
lowBit
	"Answer the index of the low order bit of this number."
	<omniUnenforced>
	| index |
	self = 0 ifTrue: [ ^ 0 ].
	index := 1.
	[ (self digitAt: index) = 0 ]
		whileTrue:
			[ index := index + 1 ].
	^ (self digitAt: index) lowBit + (8 * (index - 1))! !

!Integer methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/5/2013 10:57'!
quo: aNumber
	<omniUnenforced>
	"Refer to the comment in Number quo: "
	| ng quo |
	aNumber isInteger ifTrue: 
		[ng := self negative == aNumber negative == false.
		quo := (self digitDiv:
			(aNumber class == SmallInteger
				ifTrue: [aNumber abs]
				ifFalse: [aNumber])
			neg: ng) at: 1.
		^ quo normalize].
	^ aNumber adaptToInteger: self andSend: #quo:! !

!Integer methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 22:07'!
timesRepeat: aBlock
	"Evaluate the argument, aBlock, the number of times represented by the 
	receiver."

	<omniUnenforced>
	| count |
	count := 1.
	[count <= self]
		whileTrue: 
			[aBlock value.
			count := count + 1]! !

!BlockContext methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 22:08'!
tempAt: index 
	"Refer to the comment in ContextPart|tempAt:."

	<omniUnenforced>
	^ home at: index! !

!BlockContext methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 22:07'!
tempAt: index put: value 
	"Refer to the comment in ContextPart|tempAt:put:."

	<omniUnenforced>
	^ home at: index put: value! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 12:59'!
at: index 
	"Primitive. Assumes receiver is indexable. Answer the value of an 
	indexable element in the receiver. Fail if the argument index is not an 
	Integer or is out of bounds. Essential. See Object documentation 
	whatIsAPrimitive."

	<primitive: 60>
	<omniReplacement: #At:>
	index isInteger ifTrue:
		[self class isVariable
			ifTrue: [self errorSubscriptBounds: index]
			ifFalse: [self errorNotIndexable]].
	index isNumber
		ifTrue: [^self at: index asInteger]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:00'!
at: index put: value
	"Primitive. Assumes receiver is indexable. Store the argument value in 
	the indexable element of the receiver indicated by index. Fail if the 
	index is not an Integer or is out of bounds. Or fail if the value is not of 
	the right type for this kind of collection. Answer the value that was 
	stored. Essential. See Object documentation whatIsAPrimitive."

	<primitive: 61>
	<omniReplacement: #At:put:>
	index isInteger ifTrue:
		[self class isVariable
			ifTrue: [(index >= 1 and: [index <= self size])
					ifTrue: [self errorImproperStore]
					ifFalse: [self errorSubscriptBounds: index]]
			ifFalse: [self errorNotIndexable]].
	index isNumber
		ifTrue: [^self at: index asInteger put: value]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:17'!
basicAt: index
	"Primitive. Assumes receiver is indexable. Answer the value of an 
	indexable element in the receiver. Fail if the argument index is not an 
	Integer or is out of bounds. Essential. Do not override in a subclass. See 
	Object documentation whatIsAPrimitive."

	<primitive: 60>
	<omniReplacement: #BasicAt:>
	index isInteger ifTrue: [self errorSubscriptBounds: index].
	index isNumber
		ifTrue: [^self basicAt: index asInteger]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:17'!
basicAt: index put: value
	"Primitive. Assumes receiver is indexable. Store the second argument 
	value in the indexable element of the receiver indicated by index. Fail 
	if the index is not an Integer or is out of bounds. Or fail if the value is 
	not of the right type for this kind of collection. Answer the value that 
	was stored. Essential. Do not override in a subclass. See Object 
	documentation whatIsAPrimitive."

	<primitive: 61>
	<omniReplacement: #BasicAt:put:>
	index isInteger
		ifTrue: [(index >= 1 and: [index <= self size])
					ifTrue: [self errorImproperStore]
					ifFalse: [self errorSubscriptBounds: index]].
	index isNumber
		ifTrue: [^self basicAt: index asInteger put: value]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 5/28/2013 11:39'!
halt
	"This is the typical message to use for inserting breakpoints during 
	debugging. It behaves like halt:, but does not call on halt: in order to 
	avoid putting this message on the stack. Halt is especially useful when 
	the breakpoint message is an arbitrary one."

	<omniUnenforced>
	Halt now! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:00'!
instVarAt: index 
	"Primitive. Answer a fixed variable in an object. The numbering of the 
	variables corresponds to the named instance variables. Fail if the index 
	is not an Integer or is not the index of a fixed variable. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 73>
	<omniReplacement: #InstVarAt:>
	"Access beyond fixed variables."
	^self basicAt: index - self class instSize		! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:00'!
instVarAt: anInteger put: anObject 
	"Primitive. Store a value into a fixed variable in the receiver. The 
	numbering of the variables corresponds to the named instance variables. 
	Fail if the index is not an Integer or is not the index of a fixed variable. 
	Answer the value stored as the result. Using this message violates the 
	principle that each object has sovereign control over the storing of 
	values into its instance variables. Essential. See Object documentation 
	whatIsAPrimitive."

	<primitive: 74>
	<omniReplacement: #InstVarAt:put:>
	"Access beyond fixed fields"
	^self basicAt: anInteger - self class instSize put: anObject! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:00'!
shallowCopy
	"Answer a copy of the receiver which shares the receiver's instance variables. It should never be overridden. I'm invoked from the copy template method. Subclasses that need to specialize the copy should specialize the postCopy hook method."
	
	| class newObject index |
	<omniReplacement: #ShallowCopy>
	<primitive: 148>
	class := self class.
	class isVariable
		ifTrue: 
			[index := self basicSize.
			newObject := class basicNew: index.
			[index > 0]
				whileTrue: 
					[newObject basicAt: index put: (self basicAt: index).
					index := index - 1]]
		ifFalse: [newObject := class basicNew].
	index := class instSize.
	[index > 0]
		whileTrue: 
			[newObject instVarAt: index put: (self instVarAt: index).
			index := index - 1].
	^ newObject! !

!Object methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 7/18/2012 22:08'!
subclassResponsibility
	"This message sets up a framework for the behavior of the class' subclasses.
	Announce that the subclass should have implemented this message."
	<omniUnenforced>
	SubclassResponsibility signalFor: thisContext sender selector! !

!ReadStream methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:07'!
next
	"Primitive. Answer the next object in the Stream represented by the
	receiver. Fail if the collection of this stream is not an Array or a String.
	Fail if the stream is positioned at its end, or if the position is out of
	bounds in the collection. Optional. See Object documentation
	whatIsAPrimitive."

	<primitive: 65>
	<omniReplacement: #Next>
	position >= readLimit
		ifTrue: [^nil]
		ifFalse: [^collection at: (position := position + 1)]! !

!String methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 6/4/2013 13:10'!
replaceFrom: start to: stop with: replacement startingAt: repStart
	"Primitive. This destructively replaces elements from start to stop in the receiver starting at index, repStart, in the collection, replacement. Answer the receiver. Range checks are performed in the primitive only. Optional. See Object documentation whatIsAPrimitive."
	
	<primitive: 105>
	<omniReplacement: #ReplaceFrom:to:with:startingAt:>
	super replaceFrom: start to: stop with: replacement startingAt: repStart! !

!HashTableSizes class methodsFor: '*O-Base-Overrides' stamp: 'StefanMarr 4/30/2013 17:38'!
atLeast: lowerLimit
	"This is a helper that is considered immutable, and we make it unenforced as a performance optimization"
	<omniUnenforced>
	"Answer the next good size >= lowerlimit.
	If lowerLimit is larger than the largest known good prime,
	just make it odd."

	| primes low mid high prime |
	primes := self sizes.
	low := 1.
	high := primes size.
	lowerLimit > (primes at: high)
		ifTrue:
			[ ^ lowerLimit even
				ifTrue: [ lowerLimit + 1 ]
				ifFalse: [ lowerLimit ] ].
	[ mid := (high - low) // 2 + low.
	prime := primes at: mid.
	prime < lowerLimit
		ifTrue: [ low := mid ]
		ifFalse: [ high := mid ].
	high - low <= 1
		ifTrue: [ ^ primes at: high ] ] repeat! !
